from abc import ABC, abstractmethod
from typing import AsyncIterable, Generic, TypeVar
from uuid import UUID

import websockets

from kilroy_ws_server_py_sdk.protocol import (
    send_data_message,
    send_data_message_stream,
)
from kilroy_ws_server_py_sdk.types import JSON

DataType = TypeVar("DataType", None, JSON, AsyncIterable[JSON])


class Sender(ABC, Generic[DataType]):
    @abstractmethod
    async def send(
        self,
        websocket: websockets.WebSocketServerProtocol,
        chat_id: UUID,
        data: DataType,
    ) -> None:
        pass


class NullSender(Sender[None]):
    async def send(
        self,
        websocket: websockets.WebSocketServerProtocol,
        chat_id: UUID,
        data: None,
    ) -> None:
        return None


class SingleSender(Sender[JSON]):
    async def send(
        self,
        websocket: websockets.WebSocketServerProtocol,
        chat_id: UUID,
        data: JSON,
    ) -> None:
        await send_data_message(websocket, chat_id, data)


class StreamSender(Sender[AsyncIterable[JSON]]):
    async def send(
        self,
        websocket: websockets.WebSocketServerProtocol,
        chat_id: UUID,
        data: AsyncIterable[JSON],
    ) -> None:
        await send_data_message_stream(websocket, chat_id, data)
